# -*- coding: utf-8 -*-
"""Time Operations Package."""

from datetime import datetime

__author__ = 'Bart Grzybicki'


def now(return_type=None):
    """Return current time.

    Args:
        return_type (type): Optional argument, pass str() if you want
            to get string.

    Returns:
        date in datetime format or as a string.

    """
    if return_type == str():
        return str(datetime.now())
    else:
        return datetime.now()


def elapsed_time(time_start, time_end):
    """Return time difference.

    Args:
        time_start (datetime): start time.
        time_end (datetime): end time.

    Returns:
        datetime: difference between two times.

    """
    return time_end - time_start



# -*- coding: utf-8 -*-
"""money.pl parser module.

Currency data is retireved from the following websites:

self.site_mapping = {
    'FOREX': 'https://www.money.pl/pieniadze/forex/',
    'NBP': 'https://www.money.pl/pieniadze/nbp/srednie/',
}

"""

from collections import OrderedDict

from lxml import html

from ..funcs.string_operations import _to_unicode


class MoneyPlParser(object):
    """MoneyPlParser class.

    Classs for parsing retried currency exchange data.

    Attributes:
        moneypl_handler (object): instance of MoneyPlHandler() class.
        currency (OrderedDict): dictionary containing all currency
            exchange data.

    """

    def __init__(self, moneypl_handler, site_mapping):
        """Initialization metod.

        Args:
            moneypl_handler (object): instance of MoneyPlHandler() class.
            site_mapping (dict): A dict containing website mapping.
        """
        self.moneypl_handler = moneypl_handler
        self.currency = OrderedDict()
        for key, value in site_mapping.items():
            if value == self.moneypl_handler.url:
                self.currency[key] = {}

    def _get_text(self):
        """Return parsed data or a list with parsed data."""
        if self.moneypl_handler.page_list is not None:
            return self.moneypl_handler.page_list
        else:
            return self.moneypl_handler.page

    def parse(self):
        """Parse xpath method.

        Returns:
            Dictionary (OrderedDict) with currency exchange data.

        """
        page_raw = self._get_text()
        if type(page_raw) == list:
            # FOREX
            page = page_raw[0]
            tree = html.fromstring(page)

            post_date_forex = \
                tree.xpath('//span[@class="xqiouj-5 kFhDeu"]/text()')[0]

            eur_pln_buy = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/section[1]\
                /div[2]/div/div/div/div[1]/div[2]/div[4]/div/div[2]/div')[
                0].text_content()
            eur_pln_sell = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/section[1]\
                /div[2]/div/div/div/div[1]/div[2]/div[4]/div/div[3]/div')[
                0].text_content()

            chf_pln_buy = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/section[1]\
                /div[2]/div/div/div/div[1]/div[2]/div[3]/div/div[2]/div')[
                0].text_content()
            chf_pln_sell = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/section[1]\
                /div[2]/div/div/div/div[1]/div[2]/div[3]/div/div[3]/div')[
                0].text_content()

            gbp_pln_buy = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/section[1]\
                /div[2]/div/div/div/div[1]/div[2]/div[5]/div/div[2]/div')[
                0].text_content()
            gbp_pln_sell = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/section[1]\
                /div[2]/div/div/div/div[1]/div[2]/div[4]/div/div[3]/div')[
                0].text_content()

            usd_pln_buy = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/section[1]\
                /div[2]/div/div/div/div[1]/div[2]/div[6]/div/div[2]/div')[
                0].text_content()
            usd_pln_sell = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/section[1]\
                /div[2]/div/div/div/div[1]/div[2]/div[6]/div/div[3]/div')[
                0].text_content()

            # NBP - average
            page = page_raw[1]
            tree = html.fromstring(page)
            post_date_nbp = \
                tree.xpath('//span[@class="sc-1fexh53-0 gVFZcf"]/text()')[0]

            eur_nbp_avg = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/div[5]/div/\
                div/div[1]/div[2]/div[8]/div/div[3]/div')[
                0].text_content()
            chf_nbp_avg = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/div[5]/div/\
                div/div[1]/div[2]/div[10]/div/div[3]/div')[
                0].text_content()
            gbp_nbp_avg = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/div[5]/div/\
                div/div[1]/div[2]/div[11]/div/div[3]/div')[
                0].text_content()
            usd_nbp_avg = tree.xpath(
                '//*[@id="app"]/div/div[7]/div/div[2]/div/main/div/div[5]/div/\
                div/div[1]/div[2]/div[2]/div/div[3]/div')[
                0].text_content()

        else:
            page = page_raw

        self.currency['FOREX'] = OrderedDict()
        self.currency['NBP'] = OrderedDict()
        self.currency['FOREX']['DATA'] = _to_unicode(post_date_forex)
        self.currency['FOREX']['EUR'] = [eur_pln_buy, eur_pln_sell]
        self.currency['FOREX']['CHF'] = [chf_pln_buy, chf_pln_sell]
        self.currency['FOREX']['GBP'] = [gbp_pln_buy, gbp_pln_sell]
        self.currency['FOREX']['USD'] = [usd_pln_buy, usd_pln_sell]
        self.currency['NBP']['DATA'] = post_date_nbp
        self.currency['NBP']['EUR'] = eur_nbp_avg
        self.currency['NBP']['CHF'] = chf_nbp_avg
        self.currency['NBP']['GBP'] = gbp_nbp_avg
        self.currency['NBP']['USD'] = usd_nbp_avg

        return self.currency



# -*- coding: utf-8 -*-

"""Tests for `interface` package."""

import pytest
import mock

from requests.exceptions import RequestException

from kursywalut.interface import interface

DATA = {'FOREX': {'CHF': ['3,8007', '3,8107'],
                  'DATA': 'Dziś, 16.12.2018 11:32',
                  'EUR': ['4,2864', '4,2964'],
                  'GBP': ['4,7698', '4,2964'],
                  'USD': ['3,7924', '3,8024']},
        'NBP': {'CHF': '3,8202',
                'DATA': 'nr 243/A/NBP/2018 z dnia 14-12-2018',
                'EUR': '4,3021',
                'GBP': '4,7940',
                'USD': '3,8095'}}


@pytest.fixture
def response():
    """Sample pytest fixture.

    See more at: http://doc.pytest.org/en/latest/fixture.html
    """

    interface.get_moneypl = mock.MagicMock(return_value=DATA)

    return interface.get_moneypl()


def test_response(response, expected=DATA):
    """Sample pytest test function with the pytest fixture as an argument."""

    assert response == expected



# -*- coding: utf-8 -*-

"""Tests for MoneyPlHandler class."""

import mock

DATA = {'FOREX': {'CHF': ['3,8007', '3,8107'],
                  'DATA': 'Dziś, 16.12.2018 11:32',
                  'EUR': ['4,2864', '4,2964'],
                  'GBP': ['4,7698', '4,2964'],
                  'USD': ['3,7924', '3,8024']},
        'NBP': {'CHF': '3,8202',
                'DATA': 'nr 243/A/NBP/2018 z dnia 14-12-2018',
                'EUR': '4,3021',
                'GBP': '4,7940',
                'USD': '3,8095'}}


class TestMoneyPlHandler(object):
    """
    MoneyPlHandler class tests.
    """

    # @mock.patch('kursywalut.handlers.moneypl_handler.MoneyPlHandler')
    def test_get_moneypl(self, expected=DATA):
        """Sample pytest test function with the pytest fixture as an argument."""
        with mock.patch(
            'kursywalut.handlers.moneypl_handler.MoneyPlHandler') as MockHandler:
            MockHandler.return_value.get_moneypl.return_value = DATA
            # mock_MoneyPlHandler._get_forex = mock.ANY
            handler = MockHandler()
            request = handler.get_moneypl()

            assert request == expected
            MockHandler.assert_called()



# -*- coding: utf-8 -*-
"""Generic handler module.

It consists of GenericHandler() class, which is ihnerited by MoneyPlHandler()
class from kursywalut.handlers.moneypl_handler module. In the future, it could
be used by another handlers used to request currency data from various
websites.

"""

import logging

import requests

from ..funcs.string_operations import _to_unicode


logger = logging.getLogger(__name__)


class GenericHandler(object):
    """GenericHandler class.

    This is main handler class. All other handlers should inherit from it.

    Attributes:
        site_mapping (dict): A dict containing website mapping.
        page_list (list): A list of string containing responses from websites.
        page (str): data retrieved from website.
        url (str): URL to retrieve data from.

    """

    def __init__(self, url):
        """Initialization method."""
        self.site_mapping = None
        self.page_list = None
        self.page = None
        self.url = url
        self.download_time = None
        self.parse_time = None

    def get_webpage(self):
        """get_webpage method.

        Send the request from self.site_mapping list.

        """
        logger.debug('Data retrieving...')

        try:
            page = requests.get(self.url)
            logger.debug('Done.')
            self.page = page.content
        except requests.exceptions.RequestException as error:
            msg = _to_unicode('Błąd pobrania danych ze strony {}!'.format(self.url))
            logger.error(msg + ': ' + str(error))



# -*- coding: utf-8 -*-
"""Interface module.

Simple CLI interface to run the program.

"""

import logging

from ..funcs.string_operations import print_unicode
from .. import version
from ..handlers import MoneyPlHandler


logger = logging.getLogger(__name__)


class Config(object):
    """Class for program configuration data.

    Attributes:
        opts: sys.args passed while running the program

    """

    opts = None


def get_moneypl():
    """Get money.pl currency data.

    Returns:
        OrderedDict: Parsed data.

    """
    print_unicode('Getting data from website.')
    mpl = MoneyPlHandler()
    data = mpl.get_moneypl()
    print_unicode('data download time: {}'.format(mpl.download_time))
    print_unicode('parse time: {}'.format(mpl.parse_time))
    return data


def pretty_print_data(data):
    """Format received data.

    Args:
        data (ordereddict): OrderedDict to be printed.

    """
    detail_str = ''
    for key, value in data.items():
        if key == 'FOREX':
            print_unicode(key + u'\tkupno\tsprzedaż\n')
        elif key == 'NBP':
            print_unicode(key + u'\tkurs średni\n')

        for name, detail in value.items():
            if type(detail) == list:
                for item in detail:
                    detail_str = detail_str + '\t' + item
            else:
                detail_str = detail
            if name == 'DATA':
                tab = '\t'
            else:
                tab = ''
            if key == 'FOREX':
                print_unicode(name + u'{}'.format(tab) + detail_str)
            elif key == 'NBP':
                print_unicode(name + u'\t' + detail_str)
            detail_str = ''
        print_unicode('')


def display_header():
    """Display header."""
    print_unicode(u'##################')
    print_unicode(u'$ KursyWalut ' + version.__version__ + u' $')
    print_unicode(u'##################\n')


def run(*args):
    """Run the program."""
    Config.opts = args
    display_header()
    data = get_moneypl()
    print_unicode('')
    pretty_print_data(data)



# -*- coding: utf-8 -*-
"""String Operations Package."""

import six

__author__ = 'Bart Grzybicki'


def _to_unicode(value):
    """Convert value to unicode.

    Args:
        value (str): Value to be converted to unicode.

    Returns:
        string (Python 3) or unicode (Python 2).

    """
    return six.text_type(value)


def print_unicode(string):
    """Print unicode function.

    Prints string (Python 3) or string converted to unicode (Python 2).

    """
    print(_to_unicode(string))



# -*- coding: utf-8 -*-
"""Handlers Package."""


from .generic_handler import GenericHandler
from .moneypl_handler import MoneyPlHandler

__author__ = 'Bart Grzybicki'


__all__ = ('GenericHandler', 'MoneyPlHandler')



# -*- coding: utf-8 -*-
"""Top-level package for KursyWalut."""

import logging

from .version import __version__
from . import funcs, handlers, parsers, interface


__author__ = 'Bart Grzybicki'
__email__ = 'bgrzybicki@gmail.com'

__all__ = ('__version__')


logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

# # create console handler
console = logging.StreamHandler()
console.setLevel(logging.INFO)

# create a file handler
handler = logging.FileHandler('kursywalut.log')
handler.setLevel(logging.WARN)

# set logging format
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
handler.setFormatter(formatter)
console.setFormatter(formatter)

# add the handler to the logger
logger.addHandler(handler)
logger.addHandler(console)



# -*- coding: utf-8 -*-
"""money.pl handler module.

Prepared requests are being passed to the get_webpage() method
inherited from GenericHandler class. In the next receved response is passed
over to the MoneyPlParser().parse() method. In the last step parsed data
is returned by the get_moneypl() method.

"""

import logging

from ..funcs.time_operations import now, elapsed_time
from ..parsers.moneypl_parser import MoneyPlParser
from .generic_handler import GenericHandler


logger = logging.getLogger(__name__)


class MoneyPlHandler(GenericHandler):
    """MoneyPlHandler class.

    Attributes:
        site_mapping (dict): A dict containing website mapping.
        page_list (list): A list of string containing responses from websites.
        page (str): data retrieved from website.
        url (str): URL to retrieve data from.

        url_forex (str): URL for FOREX currency exchange.
        url_nbp (str): URL for NBP currency exchange.

    """

    def __init__(self):
        """Initialization method."""
        super(MoneyPlHandler, self).__init__(None)
        self.site_mapping = {
            'FOREX': 'https://www.money.pl/pieniadze/forex/',
            'NBP': 'https://www.money.pl/pieniadze/nbp/srednie/',
        }
        self.url_forex = self.site_mapping['FOREX']
        self.url_nbp = self.site_mapping['NBP']
        self.page_list = []

    def _get_forex(self):
        """_get_forex method.

        Send request for FOREX data and add parsed data to the list.

        """
        logger.debug(self._get_forex.__name__)
        self.url = self.url_forex
        self.get_webpage()
        self.page_list.append(self.page)
        self.parser = MoneyPlParser(self, self.site_mapping)

    def _get_nbp(self):
        """_get_nbp method.

        Send request for NBP data and add parsed data to the list.

        """
        logger.debug(self._get_nbp.__name__)
        self.url = self.url_nbp
        self.get_webpage()
        self.page_list.append(self.page)

    def get_moneypl(self):
        """get_moneypl method.

        Send requests for FOREX and NBP data and parse both responses.

        Returns:
            Parsed data as OrderedDict.

        """
        logger.debug(self.get_moneypl.__name__)
        self.page_list = []
        start = now()
        self._get_forex()
        self._get_nbp()
        self.download_time = elapsed_time(start, now())
        start = now()
        self.data_parsed = self.parser.parse()
        self.parse_time = elapsed_time(start, now())
        logger.debug('self.data_parsed={}'.format(self.data_parsed))

        return self.data_parsed
